@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2019 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,6 @@ from abc import ABCMeta, abstractmethod
 
 import shutil
 import yaml
-import json
 import minidb
 import logging
 
@@ -57,6 +56,7 @@ DEFAULT_CONFIG = {
             'line_length': 75,
             'details': True,
             'footer': True,
+            'minimal': False,
         },
 
         'html': {
@@ -78,6 +78,7 @@ DEFAULT_CONFIG = {
             'method': 'smtp',
             'smtp': {
                 'host': 'localhost',
+                'user': '',
                 'port': 25,
                 'starttls': True,
                 'keyring': True,
@@ -102,6 +103,10 @@ DEFAULT_CONFIG = {
             'bot_token': '',
             'chat_id': '',
         },
+        'slack': {
+            'enabled': False,
+            'webhook_url': '',
+        },
         'mailgun': {
             'enabled': False,
             'region': 'us',
@@ -113,6 +118,13 @@ DEFAULT_CONFIG = {
             'subject': '{count} changes: {jobs}'
         },
     },
+
+    'job_defaults': {
+        'all': {},
+        'shell': {},
+        'url': {},
+        'browser': {}
+    }
 }
 
 
@@ -277,16 +289,7 @@ class BaseYamlFileStorage(BaseTextualFileStorage, metaclass=ABCMeta):
         filename = args[0]
         if filename is not None and os.path.exists(filename):
             with open(filename) as fp:
-                return yaml.load(fp)
-
-
-class BaseJsonFileStorage(BaseTextualFileStorage, metaclass=ABCMeta):
-    @classmethod
-    def parse(cls, *args):
-        filename = args[0]
-        if filename is not None and os.path.exists(filename):
-            with open(filename) as fp:
-                return json.load(fp)
+                return yaml.load(fp, Loader=yaml.SafeLoader)
 
 
 class YamlConfigStorage(BaseYamlFileStorage):
@@ -298,15 +301,6 @@ class YamlConfigStorage(BaseYamlFileStorage):
             yaml.dump(self.config, fp, default_flow_style=False)
 
 
-class JsonConfigStorage(BaseJsonFileStorage):
-    def load(self, *args):
-        self.config = merge(self.parse(self.filename) or {}, copy.deepcopy(DEFAULT_CONFIG))
-
-    def save(self, *args):
-        with open(self.filename, 'w') as fp:
-            json.dump(self.config, fp, default_flow_style=False)
-
-
 class UrlsYaml(BaseYamlFileStorage, UrlsBaseFileStorage):
 
     @classmethod
@@ -314,7 +308,7 @@ class UrlsYaml(BaseYamlFileStorage, UrlsBaseFileStorage):
         filename = args[0]
         if filename is not None and os.path.exists(filename):
             with open(filename) as fp:
-                return yaml.load_all(fp)
+                return [JobBase.unserialize(job) for job in yaml.load_all(fp, Loader=yaml.SafeLoader) if job is not None]
 
     def save(self, *args):
         jobs = args[0]
@@ -325,21 +319,7 @@ class UrlsYaml(BaseYamlFileStorage, UrlsBaseFileStorage):
 
     def load(self, *args):
         with open(self.filename) as fp:
-            return [JobBase.unserialize(job) for job in yaml.load_all(fp) if job is not None]
-
-
-class UrlsJson(BaseJsonFileStorage, UrlsBaseFileStorage):
-    def save(self, *args):
-        jobs = args[0]
-        print('Saving updated list to %r' % self.filename)
-
-        with open(self.filename, 'w') as fp:
-            yaml.dump_all([job.serialize() for job in jobs], fp, default_flow_style=False)
-
-    def load(self, *args):
-        with open(self.filename) as fp:
-            json_data = fp.read()
-            return [JobBase.unserialize(job) for job in json.loads(json_data)['urls'] if job is not None]
+            return [JobBase.unserialize(job) for job in yaml.load_all(fp, Loader=yaml.SafeLoader) if job is not None]
 
 
 class UrlsTxt(BaseTxtFileStorage, UrlsBaseFileStorage):
@@ -415,7 +395,7 @@ class CacheDirStorage(CacheStorage):
     def load(self, job, guid):
         filename = self._get_filename(guid)
         if not os.path.exists(filename):
-            return None, None
+            return None, None, None, None
 
         try:
             with open(filename) as fp:
@@ -426,7 +406,7 @@ class CacheDirStorage(CacheStorage):
 
         timestamp = os.stat(filename)[stat.ST_MTIME]
 
-        return data, timestamp, None
+        return data, timestamp, None, None
 
     def save(self, job, guid, data, timestamp, etag=None):
         # Timestamp and ETag are always ignored
@@ -478,6 +458,20 @@ class CacheMiniDBStorage(CacheStorage):
 
         return None, None, 0, None
 
+    def get_history_data(self, guid, count=1):
+        history = {}
+        if count < 1:
+            return history
+        for data, timestamp in CacheEntry.query(self.db, CacheEntry.c.data // CacheEntry.c.timestamp,
+                                                order_by=minidb.columns(CacheEntry.c.timestamp.desc, CacheEntry.c.tries.desc),
+                                                where=(CacheEntry.c.guid == guid)
+                                                & ((CacheEntry.c.tries == 0) | (CacheEntry.c.tries == None))):  # noqa
+            if data not in history:
+                history[data] = timestamp
+                if len(history) >= count:
+                    break
+        return history
+
     def save(self, job, guid, data, timestamp, tries, etag=None):
         self.db.save(CacheEntry(guid=guid, timestamp=timestamp, data=data, tries=tries, etag=etag))
         self.db.commit()
